perl -I../lib 00-errflow_check-run.t 1
perl -I../lib 00-errflow_dire-run.t 1
perl -I../lib 00-errflow_mix1-run.t 1
perl -I../lib 00-errflow_mix2-run.t 1
perl -I../lib 00-errflow_redir-run.t 1
perl -I../lib 00-errflow_retry-run.t 1
perl -I../lib 00-errflow_unclosed-run.t 1
perl -I../lib 00-error-run.t 1
perl -I../lib 00-obj.t 1
perl -I../lib 00-simple-dire.t 1
perl -I../lib 00s-errflow_mix1-run.t 1
perl -I../lib 00s-errflow_mix2-run.t 1
perl -I../lib 00s-errflow_unclosed-run.t 1
perl -I../lib 01-simple-fail2.t 1
perl -I../lib 01-simple-fail3.t 1
perl -I../lib 01-simple-run-echo.t 1
perl -I../lib 01-simple-run-merge.t 1
perl -I../lib 01-simple-run-opts.t 1
perl -I../lib 01-simple-run-quiet.t 1
perl -I../lib 01-simple-run-stderr.t 1
perl -I../lib 01-simple-run-stdout.t 1
perl -I../lib 01-simple-run.t 1
perl -I../lib 01-simple-script=run-quiet.t 1
perl -I../lib 01-simple-script=run.t 1
perl -I../lib 01-simple-script=script.t 1
perl -I../lib 01-simple-script=simple-quiet.t 1
perl -I../lib 01-simple-script=simple.t 1
perl -I../lib 01s-simple-fail3.t 1
perl -I../lib 01s-simple-script=run.t 1
perl -I../lib 01s-simplessh-fail2.t 1
perl -I../lib 01s-simplessh-nohosts.t 1
perl -I../lib 01s-simplessh-run-opts.t 1
perl -I../lib 01s-simplessh-run.t 1
perl -I../lib 01s-simplessh2-quiet.t 1
perl -I../lib 01s-simplessh2-run.t 1
perl -I../lib 01s-simplessh2-run2.t 1
perl -I../lib 01s-simplessh2-stderr.t 1
perl -I../lib 01s-simplessh2-stdout.t 1
perl -I../lib 02-special_characters-run.t 1
perl -I../lib 02-special_characters-script=run.t 1
perl -I../lib 10-fail-run-continue.t 1
perl -I../lib 10-fail-run-display.t 1
perl -I../lib 10-fail-run.t 1
perl -I../lib 10-fail-script=run-continue.t 1
perl -I../lib 10-fail-script=run-display.t 1
perl -I../lib 10-fail-script=run.t 1
perl -I../lib 20-check-run.t 1
perl -I../lib 20-check-script=run.t 1
perl -I../lib 30-retry-run.t 1
perl -I../lib 30-retry-script=run.t 1
perl -I../lib 30-retry-script=script.t 1
perl -I../lib 30-retry_sleep-run.t 1
perl -I../lib 31-retry_fail-run-continue.t 1
perl -I../lib 31-retry_fail-run-display.t 1
perl -I../lib 31-retry_fail-run.t 1
perl -I../lib 31-retry_fail-script=run-continue.t 1
perl -I../lib 31-retry_fail-script=run-display.t 1
perl -I../lib 31-retry_fail-script=run.t 1
perl -I../lib 40-if-run.t 1
perl -I../lib 40-if-script=run.t 1
perl -I../lib 41-for-run.t 1
perl -I../lib 41-for-script=run.t 1
perl -I../lib 42-until-run.t 1
perl -I../lib 42-until-script=run.t 1
perl -I../lib 43-while-run.t 1
perl -I../lib 43-while-script=run.t 1
perl -I../lib 50-for_fail-run-continue.t 1
perl -I../lib 50-for_fail-run.t 1
perl -I../lib 50-for_fail-script=run.t 1
perl -I../lib 50-for_fail-script=script.t 1
perl -I../lib 60-check_retry-run.t 1
perl -I../lib 60-check_retry-script=run.t 1
perl -I../lib 60-check_retry-script=script.t 1
perl -I../lib 61-check_fail-run.t 1
perl -I../lib 62-check_succ-run-continue.t 1
perl -I../lib 62-check_succ-run.t 1
perl -I../lib 70-alt-run.t 1
perl -I../lib 70-alt-script=run.t 1
perl -I../lib 70-alt2-run.t 1
perl -I../lib 71-alt_retry-run.t 1
perl -I../lib 71-alt_retry-script=run.t 1
perl -I../lib 71-alt_retry-script=script.t 1
perl -I../lib 72-alt_retry-script.t 1
perl -I../lib 90-alt_retry-script.t 1
perl -I../lib 90-fail-script.t 1
perl -I../lib 90-for_fail-script.t 1
perl -I../lib 90-simple-script-quiet.t 1
perl -I../lib 90-simple-script-stderr.t 1
perl -I../lib 90-simple-script-stdout.t 1
perl -I../lib 90-simple-script.t 1
perl -I../lib 91-long-run.t 1
perl -I../lib ZZ-cleanup-run.t 1
perl -I../lib _version.t 1
